"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const component_1 = __importDefault(require("../component"));
const query_1 = require("../compiler/query");
const prop_filler_1 = __importDefault(require("../compiler/prop-filler"));
const component_filler_props_1 = __importDefault(require("../component/component-filler-props"));
const registrar_1 = __importDefault(require("../compiler/registrar"));
const globals_1 = __importDefault(require("../globals"));
const access_1 = __importDefault(require("../access"));
class Action extends component_1.default {
    constructor() {
        super(...arguments);
        this.className = 'Action';
    }
    create(props) {
        super.create(props);
        // For mocking
        this._registrar = registrar_1.default;
        this._globals = globals_1.default;
        this._access = access_1.default;
        this._componentFillerProps = new component_filler_props_1.default();
        this._fillerProps = null;
        this.set({
            schema: {
                component: 'Form',
                fields: [
                    {
                        name: 'if',
                        component: 'WhereField',
                    },
                    {
                        name: 'actions',
                        component: 'Field',
                    },
                    {
                        name: 'else',
                        component: 'Field',
                    },
                    {
                        name: 'layer',
                        component: 'TextField',
                    },
                    {
                        name: 'detached',
                        component: 'BooleanField',
                    },
                ],
            },
        });
    }
    // Abstract method
    // async act(/* props */) {}
    _fill(prop, preventQuery, customizer) {
        const propFiller = new prop_filler_1.default(this._fillerProps);
        // Fill with props from component first so that we define default values in the component like
        // {{fields.to.value}} that are then filled via the second fill.
        prop = propFiller.fill(prop, preventQuery, customizer);
        prop = propFiller.fill(prop, preventQuery); // Yes, this duplicate is needed!
        return prop;
    }
    _getFilled(names, preventQuery, customizer) {
        const prop = super.get(names);
        // When names are undefined, we need to prevent props being considered a query as a nested "if"
        // attribute is likely to contain an operator
        const prevQuery = names === undefined ? true : preventQuery;
        return prop === undefined
            ? undefined
            : this._fill(prop, prevQuery, customizer);
    }
    get(names, preventQuery, customizer) {
        return this._getFilled(names, preventQuery, customizer);
    }
    _setFillerProps(props) {
        // getFillerProps() wraps the props in a Getter so that the values can be retrieved dynamically,
        // allowing for things like retrieving data from deely nested components.
        this._fillerProps = this._componentFillerProps.getFillerProps(Object.assign(Object.assign({}, props), { self: this }));
    }
    _setWhereProps(where, props) {
        // getWhereProps() resolves all the properties in the query and populates _whereProps. This
        // allows us to dynamically query data in deeply nested components.
        this._whereProps = this._componentFillerProps.getWhereProps(where, Object.assign(Object.assign({}, props), { self: this }));
    }
    run(props) {
        return __awaiter(this, void 0, void 0, function* () {
            this._setFillerProps(props);
            const where = this.get('if', true);
            let shouldRun = true;
            let actions = null;
            if (where) {
                this._setWhereProps(where, props);
                const filtered = (0, query_1.filter)([this._whereProps], where);
                if (filtered.length === 0) {
                    // Condition failed
                    if (this.get('else')) {
                        actions = this.get('else');
                    }
                    else {
                        // The condition failed and there is nothing to execute
                        shouldRun = false;
                    }
                }
                else {
                    // Condition met
                    actions = this.get('actions');
                }
            }
            else {
                actions = this.get('actions');
            }
            if (shouldRun) {
                if (actions) {
                    let args = null;
                    for (const i in actions) {
                        if (!args && props && props.arguments) {
                            args = props.arguments;
                        }
                        args = yield actions[i].run(Object.assign(Object.assign({}, props), { arguments: args, parent: this, context: props && props.context }));
                    }
                    return args;
                }
                else {
                    return this.act(props);
                }
            }
        });
    }
}
exports.default = Action;
