"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const action_1 = __importDefault(require("./action"));
// FUTURE: enhance the Event Listener construct so that you can just specify a function inline?
// e.g.
//   actions: [
//     {
//       component: 'JavaScript',
//       function: () => 'foo'
//     }
//   ]
//
// could become:
//
//   actions: [
//     () => 'foo'
//   ]
//
//
// Behind the scenes, we'd probably want to convert these inline functions in JavaScript action
// objects so that they plug into the existing action constructs
class JavaScript extends action_1.default {
    constructor() {
        super(...arguments);
        this.className = 'JavaScript';
    }
    create(props) {
        super.create(props);
        this.set({
            schema: {
                component: 'Form',
                fields: [
                    {
                        name: 'function',
                        component: 'Field',
                        required: true,
                    },
                ],
            },
        });
    }
    // Note: this code is intentionally commented out as it introduces the risk of XSS attacks. This
    // code should REMAIN commented out in MSON, but is left here as something that can be enabled by
    // forks. It also serves as a reminder why we don't deserialize JS in MSON. See
    // https://github.com/redgeoff/mson/blob/main/DESIGN.md#why-doesnt-mson-support-custom-js-in-template-parameters
    // for a more in-depth explanation
    //
    // set(props) {
    //   if (props.function !== undefined && typeof props.function !== 'function') {
    //     // Note: use of `new Function` doesn't allow access to local scope:
    //     // https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Function/Function
    //     props = {
    //       ...props,
    //       function: new Function(
    //         'props',
    //         `const f = ${props.function.toString()}; return f(props);`
    //       ),
    //     };
    //   }
    //
    //   super.set(props);
    // }
    act(props) {
        return __awaiter(this, void 0, void 0, function* () {
            const fun = this.get('function');
            return yield fun(Object.assign(Object.assign({}, props), { globals: this._globals, session: this._componentFillerProps._getSession(), self: this }));
        });
    }
}
exports.default = JavaScript;
