"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Getter = void 0;
const globals_1 = __importDefault(require("../globals"));
const registrar_1 = __importDefault(require("../compiler/registrar"));
const query_to_props_1 = __importDefault(require("../component/query-to-props"));
const get_1 = __importDefault(require("lodash/get"));
class Getter {
    constructor({ action, component, args, globals, self }) {
        this._action = action;
        this._component = component;
        this._args = args;
        this._globals = globals;
        this._self = self;
    }
    get(name) {
        const names = name.split('.');
        const firstName = names[0];
        let item;
        let isComponent = false;
        switch (firstName) {
            case 'action':
                item = this._action;
                break;
            case 'arguments':
                item = this._args;
                break;
            case 'globals':
                item = this._globals;
                break;
            case 'self':
                item = this._self;
                break;
            default:
                isComponent = true;
                item = this._component;
                break;
        }
        if (item === undefined) {
            return undefined;
        }
        else if (names.length > 1 || isComponent) {
            let childName = name;
            if (!isComponent) {
                names.shift(); // remove first name
                childName = names.join('.');
            }
            // Is the item a getter?
            if (item.get && typeof item.get === 'function') {
                return item.get(childName);
            }
            else {
                // Item is an object
                return (0, get_1.default)(item, childName);
            }
        }
        else {
            return item;
        }
    }
}
exports.Getter = Getter;
class ComponentFillerProps {
    constructor() {
        // For mocking
        this._registrar = registrar_1.default;
    }
    _getSession() {
        const reg = this._registrar;
        return reg.client ? reg.client.user.getSession() : undefined;
    }
    // TODO: refactor so that we just expose the globals component and a session property. This way,
    // we don't process the properties until they are needed. The session property can probably be on
    // globals and simply return a session when it is retrieved, e.g. see FormEditor and the
    // definition property for a similar example.
    _getGlobals() {
        return Object.assign({ session: this._getSession() }, globals_1.default.get());
    }
    getFillerProps(props) {
        // Wrap in a Getter so that we expose a single get() to PropFiller
        return new Getter({
            action: props && props.parent,
            component: props && props.component,
            args: props && props.arguments,
            globals: this._getGlobals(),
            self: props && props.self,
        });
    }
    getWhereProps(where, props) {
        // Wrap in a Getter so that we expose a single get() to queryToProps()
        const component = new Getter({
            action: props.parent,
            component: props.component,
            args: props.arguments,
            globals: this._getGlobals(),
            self: props && props.self,
        });
        return (0, query_to_props_1.default)(where, component);
    }
}
exports.default = ComponentFillerProps;
