"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const wrapped_component_1 = __importDefault(require("./wrapped-component"));
const registrar_1 = __importDefault(require("../compiler/registrar"));
// Allows you to define to a component in MSON, which isn't built until the component is
// instantiated
class MSONComponent extends wrapped_component_1.default {
    constructor() {
        super(...arguments);
        this.className = 'MSONComponent';
    }
    create(props) {
        super.create(props);
        this.set({
            schema: {
                component: 'Form',
                fields: [
                    {
                        name: 'definition',
                        component: 'Field',
                        required: true,
                    },
                ],
            },
        });
        // We want the class name to be that of the wrapped component
        this._preserveClassName = false;
        // For mocking
        this._registrar = registrar_1.default;
    }
    // Set the compiler so that we can have separate compiler spaces per list of components
    _setCompiler(compiler) {
        this._compiler = compiler;
    }
    _getCompiler() {
        const compiler = this._compiler ? this._compiler : this._registrar.compiler;
        if (compiler) {
            return compiler;
        }
        else {
            throw new Error('compiler has not been registered with registrar');
        }
    }
    set(props) {
        super.set(props);
        if (props.definition !== undefined) {
            const compiler = this._getCompiler();
            const component = compiler.newComponent(props.definition);
            this.setComponentToWrap(component);
        }
    }
}
exports.default = MSONComponent;
