"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.throwIfNotPropertyNotDefinedError = exports.queryToPropNames = void 0;
const forEach_1 = __importDefault(require("lodash/forEach"));
const set_1 = __importDefault(require("lodash/set"));
const property_not_defined_error_1 = __importDefault(require("./property-not-defined-error"));
const queryToPropNames = (query, names, parentName) => {
    if (query !== null && typeof query === 'object') {
        if (Array.isArray(query)) {
            query.forEach((item) => (0, exports.queryToPropNames)(item, names, parentName));
        }
        else {
            (0, forEach_1.default)(query, (item, name) => {
                let itemParentName = parentName;
                // Not operator? TODO: actually check operators so that we support variables that begin with
                // $
                if (name.charAt(0) !== '$') {
                    itemParentName =
                        parentName === undefined ? name : parentName + '.' + name;
                }
                if (name === '$elemMatch') {
                    // We want to set the parent item and then skip the rest of the current tree so that the
                    // query can scan the entire array. We cannot determine which array element will be chosen
                    // until the query is run.
                    names[itemParentName] = true;
                }
                else {
                    (0, exports.queryToPropNames)(item, names, itemParentName);
                }
            });
        }
    }
    else {
        names[parentName] = true;
    }
};
exports.queryToPropNames = queryToPropNames;
const throwIfNotPropertyNotDefinedError = (err) => {
    if (!(err instanceof property_not_defined_error_1.default)) {
        throw err;
    }
};
exports.throwIfNotPropertyNotDefinedError = throwIfNotPropertyNotDefinedError;
// We analyze the MongoDB-style query to dynamically extract the properties from the component,
// including properties in deeply nested components. A previous design called get() on the
// components to return the values, but that does not always provide access to deeply nested
// components.
//
// TODO: is there a more efficient way of extracting these values via a callback in the query?
const queryToProps = (query, component) => {
    const props = {};
    const names = {};
    (0, exports.queryToPropNames)(query, names);
    (0, forEach_1.default)(names, (value, name) => {
        try {
            (0, set_1.default)(props, name, component.get(name));
        }
        catch (err) {
            // Swallow the error if the property is not defined. This allows us to do things like ignore
            // outdated filters, e.g. a filter for a property, which has since been deleted.
            (0, exports.throwIfNotPropertyNotDefinedError)(err);
        }
    });
    return props;
};
exports.default = queryToProps;
