"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const field_1 = __importDefault(require("./field"));
const date_field_1 = __importDefault(require("./date-field"));
const isValid_1 = __importDefault(require("date-fns/isValid"));
class TimeField extends field_1.default {
    constructor() {
        super(...arguments);
        this.className = 'TimeField';
    }
    create(props) {
        super.create(props);
        this.set({
            schema: {
                component: 'Form',
                fields: [
                    {
                        name: 'showSeconds',
                        component: 'BooleanField',
                        label: 'Show Seconds',
                        docLevel: 'basic',
                    },
                ],
            },
        });
    }
    set(props) {
        const clonedProps = Object.assign({}, props);
        // Convert Date? We store dates in epoch time so that they are compatiable across all stores.
        // Epoch time is also smaller than the ISO string and can therefore minimize the storage needed.
        if (clonedProps.value !== undefined &&
            typeof clonedProps.value !== 'number') {
            clonedProps.value = date_field_1.default.toEpochTime(clonedProps.value);
        }
        super.set(clonedProps);
    }
    // For mocking
    _toLocaleString(date) {
        const currentLocale = [];
        let options = {};
        if (!this.get('showSeconds')) {
            options = {
                hour: '2-digit',
                minute: '2-digit',
            };
        }
        return date.toLocaleTimeString(currentLocale, options);
    }
    getDisplayValue() {
        return this.isBlank()
            ? null
            : this._toLocaleString(new Date(this.getValue()));
    }
    validate() {
        super.validate();
        if (!this.isBlank()) {
            const value = this.getValue();
            const date = new Date(value);
            if (!(0, isValid_1.default)(date)) {
                this.setErr('invalid');
            }
        }
    }
}
exports.default = TimeField;
