"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const form_1 = __importDefault(require("./form"));
const field_1 = __importDefault(require("../fields/field"));
class ComponentDefinitionField extends field_1.default {
    // Leave className with the inherited value of "Field" so that the UI knows how to render (hide)
    // it. FUTURE: we may want to define something like a HiddenField that the we can use to represent
    // a Field that should not be rendered.
    //
    // className = 'ComponentDefinitionField';
    _setValue(value) {
        if (value !== null) {
            // Also allow for "componentName" as "component" is a MSON keyword and results in compilation
            value = Object.assign(Object.assign({}, value), { component: value.component ? value.component : value.componentName, componentName: undefined });
        }
        super._setValue(value);
    }
}
// Note: we use JS to define the component so that we can use `instanceof ComponentForm`
class ComponentForm extends form_1.default {
    constructor() {
        super(...arguments);
        this.className = 'ComponentForm';
    }
    create(props) {
        super.create(props);
        const fields = [
            // A JSON blob defining the component. We define a ComponentDefinitionField so that we can
            // dynamically modify the value, to workaround the fact that the "component" keyword cannot be
            // used in MSON.
            new ComponentDefinitionField({
                name: 'definition',
                label: 'Definition',
                required: true,
            }),
        ];
        // By default, lock down access
        const access = {
            form: {
                create: 'admin',
                read: 'admin',
                update: 'admin',
                archive: 'admin',
            },
        };
        this.set({
            access,
            fields,
        });
    }
}
exports.default = ComponentForm;
