"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const form_1 = __importDefault(require("./form"));
const select_field_1 = __importDefault(require("../fields/select-field"));
class FieldEditorForm extends form_1.default {
    constructor() {
        super(...arguments);
        this.className = 'FieldEditorForm';
    }
    _namesToOptions() {
        return this.constructor.componentNames.map((name) => ({
            value: name,
            label: name,
        }));
    }
    _clearFields() {
        this.eachField((field) => {
            const name = field.get('name');
            if (!this.isDefaultField(name) && name !== 'componentName') {
                this.removeField(name);
            }
        });
        this._emitChangeToFields(null);
    }
    _copySchemaFields(schema) {
        const fields = [];
        schema.eachField((field) => {
            const name = field.get('name');
            if (!schema.isDefaultField(name) && field.get('docLevel') === 'basic') {
                this.addField(field);
                fields.push(field);
            }
        });
        this._emitChangeToFields(fields);
    }
    _adjustFields(values) {
        // Use the component schema to display fields to the user
        // Clear any previous fields that are not default fields or componentName
        this._clearFields();
        if (values.componentName) {
            const compiler = this._registrar.compiler;
            this._component = compiler.newComponent({
                component: values.componentName,
            });
            const schema = new form_1.default();
            this._component.buildSchemaForm(schema, compiler);
            this._copySchemaFields(schema);
            // Preserve any applicable values from the previous component
            Object.entries(values).forEach(([name, value]) => {
                if (this.hasField(name)) {
                    this.getField(name).setValue(value);
                }
            });
        }
    }
    create(props) {
        super.create(props);
        this.set({
            fields: [
                new select_field_1.default({
                    name: 'componentName',
                    label: 'Field',
                    options: this._namesToOptions(),
                }),
            ],
        });
        // Adjust the fields when the componentName changes
        this.getField('componentName').on('value', () => this._adjustFields(this.getValues()));
    }
    setValues(values) {
        // Is the componentName changing?
        if (!this._component ||
            (values.componentName &&
                this._component.getClassName() !== values.componentName)) {
            this._adjustFields(Object.assign({}, this.getValues(), {
                componentName: values.componentName,
            }));
        }
        super.setValues(values);
    }
}
exports.default = FieldEditorForm;
FieldEditorForm.componentNames = [
    'AddressField',
    'BooleanField',
    'ButtonField',
    'ChainedSelectField',
    'ChainedSelectListField',
    'CityField',
    // 'CollectionField',
    // 'CompositeField',
    'CountryField',
    'DateField',
    'EmailField',
    // 'FormField',
    'IdField',
    'IntegerField',
    // 'ListField',
    'MoneyField',
    'NumberField',
    'PasswordField',
    'PersonFullNameField',
    'PersonNameField',
    'PhoneField',
    'ProvinceField',
    'ReCAPTCHAField',
    // 'RegExpField',
    'SelectField',
    'SelectListField',
    'StateField',
    'Text',
    'TextField',
    'TextListField',
    'TimeField',
    'URLField',
];
