"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const form_1 = __importDefault(require("./form"));
const field_1 = __importDefault(require("../fields/field"));
const form_field_1 = __importDefault(require("../fields/form-field"));
// e.g.
// access: {
//   form: {
//     create: 'role2',
//     read: [ 'role1', 'role2' ],
//     update: [ 'role1', 'role2' ],
//     archive: 'role2'
//   },
//
//   fields: {
//     firstName: {
//       create: 'role2',
//       read: [ 'role1', 'role2' ],
//       update: 'role2',
//       archive: 'role2'
//     }
//   }
// }
class AccessRolesField extends field_1.default {
    constructor() {
        super(...arguments);
        this.className = 'AccessRolesField';
    }
    create(props) {
        super.create(Object.assign({}, props, { allowScalar: true }));
    }
}
class AccessNode extends form_1.default {
    constructor() {
        super(...arguments);
        this.className = 'AccessNode';
    }
    create(props) {
        super.create(props);
        this.addField(new AccessRolesField({ name: 'create', label: 'Create' }));
        this.addField(new AccessRolesField({ name: 'read', label: 'Create' }));
        this.addField(new AccessRolesField({ name: 'update', label: 'Create' }));
        this.addField(new AccessRolesField({ name: 'archive', label: 'Create' }));
    }
}
class AccessFields extends form_1.default {
    constructor() {
        super(...arguments);
        this.className = 'AccessFields';
    }
    create(props) {
        super.create(props);
        this.set({
            schema: {
                component: 'Form',
                fields: [
                    {
                        name: 'fieldNames',
                        component: 'Field', // TODO: properly define
                    },
                ],
            },
        });
    }
    set(props) {
        super.set(props);
        if (props.fieldNames !== undefined) {
            this.removeFieldsExcept();
            props.fieldNames.forEach((fieldName) => {
                this.addField(new form_field_1.default({
                    name: fieldName,
                    form: new AccessNode(),
                }));
            });
        }
    }
}
class FormAccess extends form_1.default {
    constructor() {
        super(...arguments);
        this.className = 'FormAccess';
    }
    create(props) {
        super.create(props);
        this.set({
            schema: {
                component: 'Form',
                fields: [
                    {
                        name: 'fieldNames',
                        component: 'Field', // TODO: properly define
                    },
                ],
            },
        });
        this.addField(new form_field_1.default({
            name: 'form',
            label: 'Form',
            form: new AccessNode(),
        }));
        this.addField(new form_field_1.default({
            name: 'fields',
            label: 'Fields',
            form: new AccessFields(),
        }));
    }
    set(props) {
        super.set(props);
        if (props.fieldNames !== undefined) {
            this.getField('fields').getForm().set({ fieldNames: props.fieldNames });
        }
    }
}
exports.default = FormAccess;
