"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const form_1 = __importDefault(require("./form"));
const form_editor_1 = __importDefault(require("./form-editor"));
const tabs_1 = __importDefault(require("../tabs"));
const form_field_1 = __importDefault(require("../fields/form-field"));
const set_1 = __importDefault(require("../actions/set"));
const emit_1 = __importDefault(require("../actions/emit"));
const json_stringify_1 = __importDefault(require("../actions/json-stringify"));
const text_1 = __importDefault(require("../text"));
const fragment_1 = __importDefault(require("../fragment"));
const container_1 = __importDefault(require("../container"));
class FormBuilder extends form_1.default {
    constructor() {
        super(...arguments);
        this.className = 'FormBuilder';
    }
    create(props) {
        super.create(props);
        this.set({
            schema: {
                component: 'Form',
                fields: [
                    {
                        // This name is needed so that we can make sure to relay the value to the backend or any
                        // front-end component cache
                        name: 'formName',
                        component: 'TextField',
                    },
                    {
                        name: 'definition',
                        component: 'Field',
                    },
                ],
            },
            fields: [
                new tabs_1.default({
                    name: 'tabs',
                    items: [
                        {
                            name: 'edit',
                            label: 'Edit',
                            icon: 'Edit',
                        },
                        {
                            name: 'preview',
                            label: 'Preview',
                            icon: 'ViewCompact',
                        },
                        {
                            name: 'export',
                            label: 'Export',
                            icon: 'code',
                        },
                    ],
                }),
                // Note: Fragment is not a field so it will be automatically wrapped with a field and
                // accessible at `fields.export.content`
                new fragment_1.default({
                    name: 'export',
                    hidden: true,
                    items: [
                        new container_1.default({ name: 'header' }),
                        new text_1.default({ name: 'definition' }),
                        new container_1.default({ name: 'footer' }),
                    ],
                }),
                new form_field_1.default({
                    name: 'form',
                    form: new form_editor_1.default(),
                }),
            ],
            listeners: [
                {
                    event: 'fields.tabs.content.edit',
                    actions: [
                        new set_1.default({
                            name: 'fields.form.form.editable',
                            value: true,
                        }),
                        new emit_1.default({
                            event: 'hideCode',
                        }),
                    ],
                },
                {
                    event: 'fields.tabs.content.preview',
                    actions: [
                        new set_1.default({
                            name: 'fields.form.form.editable',
                            value: false,
                        }),
                        new emit_1.default({
                            event: 'hideCode',
                        }),
                    ],
                },
                {
                    event: 'fields.tabs.content.export',
                    actions: [
                        new json_stringify_1.default({
                            value: '{{definition}}',
                            space: 2,
                        }),
                        new set_1.default({
                            name: 'fields.export.content.items.definition.text',
                            value: '```js\n{{arguments}}\n```\n',
                        }),
                        new emit_1.default({
                            event: 'setCodeHidden',
                            value: {
                                hideCode: false,
                                hideForm: true,
                            },
                        }),
                    ],
                },
                {
                    event: 'setCodeHidden',
                    actions: [
                        new set_1.default({
                            name: 'fields.export.content.hidden',
                            value: '{{arguments.hideCode}}',
                        }),
                        new set_1.default({
                            name: 'fields.form.hidden',
                            value: '{{arguments.hideForm}}',
                        }),
                    ],
                },
                {
                    event: 'hideCode',
                    actions: [
                        new emit_1.default({
                            event: 'setCodeHidden',
                            value: {
                                hideCode: true,
                                hideForm: false,
                            },
                        }),
                    ],
                },
            ],
        });
    }
    _setDefinition(definition) {
        const fields = definition.fields.map((field) => {
            // Use componentName instead of component
            const value = Object.assign(Object.assign({}, field), { componentName: field.component });
            delete value.component;
            return value;
        });
        this.get('fields.form.form.fields.fields').setValue(fields);
        this.set({ formName: definition.name });
    }
    set(props) {
        super.set(Object.assign(Object.assign({}, props), { definition: undefined }));
        if (props.definition !== undefined) {
            this._setDefinition(props.definition);
        }
        // We also allow the definition to be set via the value so that both the definition and other
        // values, like id & userId, can be set simultaneously
        if (props.value !== undefined && props.value.definition !== undefined) {
            this._setDefinition(props.value.definition);
        }
    }
    _getDefinition() {
        return {
            name: this.get('formName'),
            component: 'Form',
            fields: this.get('fields.form.form.fields.fields').mapForms((form) => (Object.assign(Object.assign({}, form.getValues({ default: false })), { component: form.getValue('componentName'), componentName: undefined }))),
        };
    }
    getOne(name) {
        if (name === 'definition') {
            return this._getDefinition();
        }
        return super.getOne(name);
    }
}
exports.default = FormBuilder;
