"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const form_1 = __importDefault(require("./form"));
const collection_field_1 = __importDefault(require("../fields/collection-field"));
const factory_1 = __importDefault(require("../component/factory"));
const field_editor_form_1 = __importDefault(require("./field-editor-form"));
const utils_1 = __importDefault(require("../utils/utils"));
class FormEditor extends form_1.default {
    constructor() {
        super(...arguments);
        this.className = 'FormEditor';
    }
    create(props) {
        super.create(props);
        this.set({
            schema: {
                component: 'Form',
                fields: [
                    {
                        name: 'definition',
                        component: 'Field',
                    },
                ],
            },
            fields: [
                new collection_field_1.default({
                    name: 'fields',
                    label: 'Fields',
                    hideLabel: true,
                    maxColumns: 1,
                    skipRead: true,
                    includeExtraneous: true,
                    forbidOrder: false,
                    forbidSort: true,
                    formFactory: new factory_1.default({
                        product: () => new field_editor_form_1.default({
                            value: {
                                // Generate a unique id so that the UI has a key when displaying a list of items.
                                // We generate the id here as we don't want the rendering layer adding the ids.
                                id: utils_1.default.uuid(),
                            },
                            validators: [
                                {
                                    where: {
                                        'parent.value': {
                                            $elemMatch: {
                                                id: {
                                                    $ne: '{{fields.id.value}}',
                                                },
                                                name: '{{fields.name.value}}',
                                            },
                                        },
                                    },
                                    error: {
                                        field: 'name',
                                        error: 'must be unique',
                                    },
                                },
                            ],
                        }),
                    }),
                }),
            ],
        });
    }
    _setDefinition(definition) {
        const fields = definition.fields.map((field) => {
            // Use componentName instead of component
            const value = Object.assign(Object.assign({}, field), { componentName: field.component });
            delete value.component;
            return value;
        });
        this.get('fields.fields').setValue(fields);
    }
    set(props) {
        super.set(Object.assign(Object.assign({}, props), { definition: undefined }));
        if (props.definition !== undefined) {
            this._setDefinition(props.definition);
        }
    }
    _getDefinition() {
        return {
            component: 'Form',
            fields: this.get('fields.fields').mapForms((form) => (Object.assign(Object.assign({}, form.getValues({ default: false })), { component: form.getValue('componentName'), componentName: undefined }))),
        };
    }
    getOne(name) {
        if (name === 'definition') {
            return this._getDefinition();
        }
        return super.getOne(name);
    }
}
exports.default = FormEditor;
