"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const form_1 = __importDefault(require("./form"));
const mson_component_1 = __importDefault(require("../component/mson-component"));
class User extends form_1.default {
    constructor() {
        super(...arguments);
        this.className = 'User';
    }
    create(props) {
        super.create(props);
        // Note: we use MSONComponents as EmailField and PasswordField are uncompiled and importing from
        // ../fields would create a circular dependency.
        const fields = [
            new mson_component_1.default({
                definition: {
                    component: 'EmailField',
                    name: 'username',
                    label: 'Email',
                    required: true,
                    // in: false,
                },
            }),
            new mson_component_1.default({
                definition: {
                    component: 'PasswordField',
                    name: 'password',
                    label: 'Password',
                    // required: true, // Required by listeners when creating
                    in: false,
                    out: false,
                    hidden: true,
                    forbidSort: true,
                },
            }),
        ];
        const schema = {
            component: 'Form',
            fields: [
                {
                    name: 'roles',
                    component: 'RolesField',
                },
            ],
        };
        // By default, lock down access
        const access = {
            form: {
                create: 'admin',
                read: 'admin',
                update: 'admin',
                archive: 'admin',
            },
        };
        this.set({
            schema,
            access,
            fields,
        });
    }
}
exports.default = User;
