/// <reference types="node" />
import events from 'events';
export declare type Key = string | number | undefined | null;
declare type FullEntry<V> = [Key, V];
declare type Entry<V> = V | FullEntry<V>;
declare type ItemGenerator<V> = Generator<Entry<V>, void>;
declare type ValueGenerator<V> = Generator<V, void>;
declare type FullEntryGenerator<V> = Generator<FullEntry<V>, void>;
export declare type OnValue<V, U> = (value?: V, key?: Key, last?: boolean) => boolean | U | void;
export declare type MapOnValue<V, U> = (value?: V, key?: Key, last?: boolean) => U;
export interface Item<V> {
    key: Key;
    prevKey: Key;
    nextKey: Key;
    value: V;
}
declare type Items<V> = {
    [key: string]: Item<V>;
};
export default class Mapa<V> extends events.EventEmitter {
    private _firstKey;
    private _lastKey;
    private _length;
    protected _items: Items<V>;
    constructor();
    clear(): void;
    protected _getItem(key: Key): Item<V> | undefined;
    protected _throwKeyFalsyError(): void;
    protected _setItem(key: Key, item: Item<V>): void;
    protected _deleteItem(key: Key): void;
    has(key: Key): boolean;
    private _nullOrUndefined;
    private _getOrThrow;
    private _insertBefore;
    private _insertAfter;
    private _insert;
    private _delete;
    private _update;
    emitChange(name: string, item: Item<V>): void;
    set(key: Key, value: V, beforeKey?: Key, afterKey?: Key): void;
    get(key: Key): V;
    walkForward(key: Key, fullEntry?: boolean): ItemGenerator<V>;
    walkBackward(key: Key, fullEntry?: boolean): ItemGenerator<V>;
    values(startKey?: Key, reverse?: boolean): ValueGenerator<V>;
    entries(startKey?: Key, reverse?: boolean): FullEntryGenerator<V>;
    forEach<U>(onValue: OnValue<V, U>): void;
    each<U>(onValue: OnValue<V, U>): void;
    map<U>(onValue: MapOnValue<V, U>): U[];
    delete(key: Key): Item<V>;
    length(): number;
    previousKey(key: Key): Key;
    previous(key: Key): V;
    first(): V;
    firstKey(): Key;
    hasFirst(): boolean;
    isEmpty(): boolean;
    nextKey(key: Key): Key;
    next(key: Key): V;
    last(): V;
    lastKey(): Key;
    hasLast(): boolean;
}
export {};
