"use strict";
// Mock Firebase so that we don't have to test against the actual service
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const mapa_1 = __importDefault(require("../mapa"));
class SnapshotMock {
    constructor(docs) {
        this._docs = docs;
    }
    docChanges() {
        return this._docs.map((doc) => ({
            type: doc.type,
            doc: {
                data: () => doc.data,
            },
        }));
    }
}
class CollectionMock {
    _listenToChanges() {
        // Not currently needed by our tests, but may be used in the future.
        //
        // Simulate the response from Firebase after a write
        // this._docs.on('$change', (name, value) => {
        //   switch (name) {
        //     case 'createDoc':
        //       this.emitSnapshot([ { type: 'added', data: value.value } ])
        //       break;
        //     case 'updateDoc':
        //       this.emitSnapshot([ { type: 'modified', data: value.value } ])
        //       break;
        //     case 'deleteDoc':
        //       this.emitSnapshot([ { type: 'removed', data: value.value } ])
        //       break;
        //     default:
        //       break;
        //   }
        // });
    }
    constructor(docs) {
        this._docs = docs;
        this._listenToChanges();
    }
    orderBy() {
        return this;
    }
    emitSnapshot(docs) {
        this._onSnapshot(new SnapshotMock(docs));
    }
    emitError(err) {
        this._onError(err);
    }
    onSnapshot(onSnapshot, onError) {
        this._onSnapshot = onSnapshot;
        this._onError = onError;
        // Emit initial data
        this.emitSnapshot(this._docs.map((doc) => ({ type: 'added', data: doc.value })));
    }
    doc(id) {
        return {
            set: (doc) => __awaiter(this, void 0, void 0, function* () {
                return this._docs.set(id, doc);
            }),
        };
    }
}
class FirebaseMock {
    constructor(docs) {
        this._docs = new mapa_1.default();
        if (docs) {
            docs.forEach((doc) => this._docs.set(doc.id, doc));
        }
        this._collection = new CollectionMock(this._docs);
        this.apps = {
            length: 0,
        };
    }
    initializeApp() {
        this.apps.length = 1;
    }
    firestore() {
        return {
            settings: () => { },
            collection: () => this._collection,
        };
    }
    emitSnapshot(docs) {
        this._collection.emitSnapshot(docs);
    }
    emitError(err) {
        this._collection.emitError(err);
    }
}
exports.default = FirebaseMock;
