"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const memory_store_1 = __importDefault(require("./memory-store"));
class LocalStorageStore extends memory_store_1.default {
    constructor() {
        super(...arguments);
        this.className = 'LocalStorageStore';
    }
    _loadDocs(storeName) {
        const store = this._localStorage.getItem(storeName);
        if (store !== null) {
            const docs = JSON.parse(store);
            docs.forEach((doc) => {
                this._docs.set(doc.id, doc);
            });
        }
    }
    // For mocking
    _getLocalStorage() {
        return this._global.window.localStorage;
    }
    create(props) {
        super.create(props);
        // For mocking
        this._global = global;
        this._localStorage = this._getLocalStorage();
        this.set({
            schema: {
                component: 'Form',
                fields: [
                    {
                        name: 'storeName',
                        component: 'TextField',
                        required: true,
                    },
                ],
            },
        });
        this._loadDocs(props && props.storeName);
    }
    _saveDocs() {
        const docs = this._docs.map((doc) => doc);
        this._localStorage.setItem(this.get('storeName'), JSON.stringify(docs));
    }
    _doAndSave(promiseFactory) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield promiseFactory();
            this._saveDocs();
            return response;
        });
    }
    _createDoc(props) {
        const _super = Object.create(null, {
            _createDoc: { get: () => super._createDoc }
        });
        return __awaiter(this, void 0, void 0, function* () {
            return this._doAndSave(() => _super._createDoc.call(this, props));
        });
    }
    _updateDoc(props) {
        const _super = Object.create(null, {
            _updateDoc: { get: () => super._updateDoc }
        });
        return __awaiter(this, void 0, void 0, function* () {
            return this._doAndSave(() => _super._updateDoc.call(this, props));
        });
    }
    _archiveDoc(props) {
        const _super = Object.create(null, {
            _archiveDoc: { get: () => super._archiveDoc }
        });
        return __awaiter(this, void 0, void 0, function* () {
            return this._doAndSave(() => _super._archiveDoc.call(this, props));
        });
    }
    _restoreDoc(props) {
        const _super = Object.create(null, {
            _restoreDoc: { get: () => super._restoreDoc }
        });
        return __awaiter(this, void 0, void 0, function* () {
            return this._doAndSave(() => _super._restoreDoc.call(this, props));
        });
    }
}
exports.default = LocalStorageStore;
