"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const store_1 = __importDefault(require("./store"));
const store_mapa_1 = __importDefault(require("./store-mapa"));
const deep_clone_1 = require("../utils/deep-clone");
const orderBy_1 = __importDefault(require("lodash/orderBy"));
const query_1 = require("../compiler/query");
const reorder_1 = require("./reorder");
class MemoryStore extends store_1.default {
    constructor() {
        super(...arguments);
        this.className = 'MemoryStore';
    }
    create(props) {
        super.create(props);
        this._docs = new store_mapa_1.default();
        this._docs.on('$change', (name, value) => {
            this.emitChange(name + 'Doc', value);
        });
    }
    _numUnarchived() {
        let n = 0;
        for (const value of this._docs.values()) {
            if (!value.archivedAt) {
                n++;
            }
        }
        return n;
    }
    _createDoc({ fieldValues, id, order, reorder }) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._shouldSetToLastOrder(order, reorder)) {
                order = this._numUnarchived();
            }
            const userId = this._getUserId();
            const doc = this._buildDoc({ fieldValues, id, order, userId });
            this._docs.set(doc.id, doc, undefined, undefined, true);
            return doc;
        });
    }
    _toMongoWhere(where) {
        return (0, deep_clone_1.cloneDeepWith)(where, (doc) => {
            if (doc && doc.$iLike) {
                return { $regex: '^' + doc.$iLike.replace(/%/, ''), $options: 'i' };
            }
        });
    }
    _hasDoc({ id }) {
        return __awaiter(this, void 0, void 0, function* () {
            return this._docs.has(id);
        });
    }
    _getDoc({ id }) {
        return __awaiter(this, void 0, void 0, function* () {
            return this._docs.get(id);
        });
    }
    _getAllDocs({ where, showArchived, order }) {
        return __awaiter(this, void 0, void 0, function* () {
            // TODO: after, first, before, last and cursors in results
            if (where) {
                // Convert to a mongo expression and then filter
                where = this._toMongoWhere(where);
            }
            const docs = {
                // TODO: hasNextPage will need to change once we support pagination via after, first, etc...
                // depending on if there is still more data to get
                pageInfo: {
                    hasNextPage: false,
                },
                edges: [],
            };
            for (const doc of this._docs.values()) {
                const filtered = where === undefined ? null : (0, query_1.filter)([doc], where);
                if ((showArchived === null ||
                    showArchived === undefined ||
                    !!doc.archivedAt === showArchived) &&
                    (where === undefined || filtered.length !== 0)) {
                    docs.edges.push({
                        node: doc,
                    });
                }
            }
            if (order) {
                // Order by properties
                const names = [];
                const orders = [];
                order.forEach((order) => {
                    names.push('node.' + order[0]);
                    orders.push(order[1].toLowerCase());
                });
                docs.edges = (0, orderBy_1.default)(docs.edges, names, orders);
            }
            return docs;
        });
    }
    _updateDoc({ id, fieldValues, order }) {
        return __awaiter(this, void 0, void 0, function* () {
            // Clone the data so that we don't modify the original
            let doc = (0, deep_clone_1.cloneDeep)(this._docs.get(id));
            doc = this._setDoc({ doc, fieldValues, order });
            this._docs.set(id, doc, undefined, undefined, true);
            return doc;
        });
    }
    _archiveDoc({ id }) {
        return __awaiter(this, void 0, void 0, function* () {
            // Clone the data so that we don't modify the original
            let doc = (0, deep_clone_1.cloneDeep)(this._docs.get(id));
            doc = this._setDoc({
                doc,
                archivedAt: this._now(),
                order: reorder_1.Reorder.DEFAULT_ORDER,
            });
            this._docs.set(id, doc, undefined, undefined, true);
            return doc;
        });
    }
    _restoreDoc({ id, reorder, order }) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._shouldSetToLastOrder(order, reorder)) {
                order = this._numUnarchived();
            }
            // Clone the data so that we don't modify the original
            let doc = (0, deep_clone_1.cloneDeep)(this._docs.get(id));
            doc = this._setDoc({ doc, archivedAt: null, order });
            this._docs.set(id, doc, undefined, undefined, true);
            return doc;
        });
    }
}
exports.default = MemoryStore;
