"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const memory_store_1 = __importDefault(require("./memory-store"));
// Mock the record API so that we don't need to test against an actual server
class RecordMock {
    constructor() {
        // We use a MemoryStore that we get automatic sorting, querying, etc...
        this._docs = new memory_store_1.default();
    }
    create({ appId, componentName, fieldValues, order }) {
        return __awaiter(this, void 0, void 0, function* () {
            const doc = yield this._docs._createDoc({ fieldValues, order });
            return {
                data: {
                    createRecord: doc,
                },
            };
        });
    }
    get({ appId, componentName, id, where }) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const doc = yield this._docs.getDoc({ id, where });
                return {
                    data: {
                        record: doc,
                    },
                };
            }
            catch (err) {
                // if (err.message === 'value is missing for key ' + id) {
                throw new Error('GraphQL error: record not found');
                // } else {
                //   throw err;
                // }
            }
        });
    }
    update({ appId, componentName, id, fieldValues, order }) {
        return __awaiter(this, void 0, void 0, function* () {
            const doc = yield this._docs._updateDoc({ id, fieldValues, order });
            return {
                data: {
                    updateRecord: doc,
                },
            };
        });
    }
    archive({ appId, componentName, id }) {
        return __awaiter(this, void 0, void 0, function* () {
            const doc = yield this._docs.archiveDoc({ id });
            return {
                data: {
                    archiveRecord: doc,
                },
            };
        });
    }
    restore({ appId, componentName, id }) {
        return __awaiter(this, void 0, void 0, function* () {
            const doc = yield this._docs.restoreDoc({ id });
            return {
                data: {
                    restoreRecord: doc,
                },
            };
        });
    }
    getAll(opts) {
        return __awaiter(this, void 0, void 0, function* () {
            const docs = yield this._docs.getAllDocs(opts);
            return {
                data: {
                    records: {
                        edges: docs.edges,
                        // TODO: this will probably need adjusted once our tests support pagination
                        pageInfo: {
                            hasNextPage: false,
                        },
                    },
                },
            };
        });
    }
}
exports.default = RecordMock;
