"use strict";
// TODO: make changes real time with subscriptions
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const store_1 = __importDefault(require("./store"));
const utils_1 = __importDefault(require("../utils/utils"));
const uber_utils_1 = __importDefault(require("../utils/uber-utils"));
const registrar_1 = __importDefault(require("../compiler/registrar"));
const reorder_1 = __importStar(require("./reorder"));
class RecordStore extends store_1.default {
    constructor() {
        super(...arguments);
        this.className = 'RecordStore';
    }
    create(props) {
        super.create(props);
        // For mocking
        this._uberUtils = uber_utils_1.default;
        this._registrar = registrar_1.default;
        this.set({
            schema: {
                component: 'Form',
                fields: [
                    {
                        name: 'storeName',
                        component: 'TextField',
                        required: true,
                    },
                ],
            },
        });
    }
    set(props) {
        super.set(props);
        this._clearCache();
    }
    _tryAndHandleError(promiseFactory) {
        return __awaiter(this, void 0, void 0, function* () {
            // Disable the error handling in the parent as the error handling will be done in _request
            return promiseFactory();
        });
    }
    _request({ form, promiseFactory }) {
        return __awaiter(this, void 0, void 0, function* () {
            const appId = this._globals.get('appId');
            try {
                return yield promiseFactory(appId);
            }
            catch (err) {
                if (form) {
                    this._uberUtils.setFormErrorsFromAPIError(err, form);
                }
                else {
                    this._uberUtils.displayError(err.toString());
                }
                // We throw the error so that the entire listener chain is aborted
                throw err;
            }
        });
    }
    _createDoc({ form, fieldValues, order, reorder }) {
        return __awaiter(this, void 0, void 0, function* () {
            this._clearCache();
            if (this._shouldSetToLastOrder(order, reorder)) {
                // Reorder and get the lastOrder so that we can add the new doc to the end of the list
                const result = yield this._reorder({ form });
                order = result.lastOrder;
                fieldValues = Object.assign({}, fieldValues, { order });
            }
            const response = yield this._request({
                form,
                promiseFactory: (appId) => {
                    return this._registrar.client.record.create({
                        appId,
                        componentName: this.get('storeName'),
                        fieldValues,
                        order,
                    });
                },
            });
            return response.data.createRecord;
        });
    }
    _clearCache() {
        this._cachedQueries = {};
    }
    _inCache(opts) {
        return this._cachedQueries[JSON.stringify(opts)] ? true : false;
    }
    _addToCache(opts) {
        this._cachedQueries[JSON.stringify(opts)] = true;
    }
    _getShowArchivedWhere(showArchived) {
        if (showArchived === null) {
            return null;
        }
        else if (showArchived) {
            return { archivedAt: { $ne: null } };
        }
        else {
            return { archivedAt: null };
        }
    }
    _getAllDocs({ form, where, showArchived, after, first, before, last, order, searchString, }) {
        return __awaiter(this, void 0, void 0, function* () {
            const showArchivedWhere = this._getShowArchivedWhere(showArchived);
            where = utils_1.default.combineWheres(showArchivedWhere, where, this.get('where'));
            return this._request({
                form,
                promiseFactory: (appId) => __awaiter(this, void 0, void 0, function* () {
                    const opts = {
                        appId,
                        componentName: this.get('storeName'),
                        asArray: true,
                        where,
                        after,
                        first,
                        before,
                        last,
                        order,
                        showArchived,
                        searchString,
                    };
                    // The built-in apollo client cache cannot automatically accomodate the mutations so we use a
                    // thin layer on top to clear the cache (invalidate it) when it needs to be rebuilt. In
                    // particular this is needed when archiving/restoring and then toggling showArchived. TODO: is
                    // there a better way? Using readQuery and writeQuery at the ApolloClient layer doesn't appear
                    // to fix this issue.
                    if (!this._inCache(opts)) {
                        this._addToCache(opts);
                        opts.bypassCache = true;
                    }
                    const response = yield this._registrar.client.record.getAll(opts);
                    return response.data.records;
                }),
            });
        });
    }
    _reorder({ form, id, order }) {
        return __awaiter(this, void 0, void 0, function* () {
            let loop = true;
            const props = {
                first: this.constructor.ITEMS_PER_PAGE_DEFAULT,
                order: [['order', 'asc']],
                where: this.get('where'),
            };
            const itemsToReorder = [];
            const onReorder = (item, newOrder) => {
                // Only reorder the affected items. The item being moved will be updated afterwards
                if (item.id !== id) {
                    itemsToReorder.push({ id: item.id, order: newOrder });
                }
            };
            let i = 0;
            let destinationKey = null;
            while (loop) {
                const response = yield this.getAllDocs(props);
                for (let j = 0; j < response.edges.length; j++) {
                    const item = response.edges[j];
                    const result = reorder_1.default.reorderItem(item.node, id, order, i, destinationKey, onReorder);
                    i = result.i;
                    destinationKey = result.destinationKey;
                }
                props.after = response.pageInfo.endCursor;
                loop = response.pageInfo.hasNextPage;
            }
            if (itemsToReorder.length > 0) {
                // We have to clear the cache as the order of the items has changed.
                this._clearCache();
                // We have to update the items after we get them all or else our updates can affect the order the
                // pagination results
                for (let j = 0; j < itemsToReorder.length; j++) {
                    const { id, order } = itemsToReorder[j];
                    // Set to blank as we are using partial updates and don't need to send all the data
                    const fieldValues = {};
                    yield this._requestUpdate({ form, id, order, fieldValues });
                }
            }
            return {
                lastOrder: i,
            };
        });
    }
    _requestUpdate({ form, id, fieldValues, order }) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield this._request({
                form,
                promiseFactory: (appId) => {
                    return this._registrar.client.record.update({
                        appId,
                        componentName: this.get('storeName'),
                        id,
                        fieldValues,
                        order,
                    });
                },
            });
            return response.data.updateRecord;
        });
    }
    _updateDoc({ form, id, fieldValues, order, reorder }) {
        return __awaiter(this, void 0, void 0, function* () {
            if (reorder) {
                yield this._reorder({ form, id, order });
            }
            return this._requestUpdate({ form, id, fieldValues, order });
        });
    }
    _getDoc({ form, id, where }) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield this._request({
                form,
                promiseFactory: (appId) => {
                    return this._registrar.client.record.get({
                        appId,
                        componentName: this.get('storeName'),
                        id,
                        where,
                    });
                },
            });
            return response.data.record;
        });
    }
    getDoc(props) {
        return __awaiter(this, void 0, void 0, function* () {
            return this._getDoc(props);
        });
    }
    // TODO: refactor out after upsert is supported by API. Note: this is not a high priority as the
    // cache keeps us from hitting the API in many cases.
    _upsertDoc(props) {
        return __awaiter(this, void 0, void 0, function* () {
            const id = props.id;
            let exists = false;
            if (id) {
                try {
                    yield this._getDoc({ id });
                    exists = true;
                }
                catch (err) {
                    if (err.message === 'GraphQL error: record not found') {
                        // Do nothing as record doesn't exist
                    }
                    else {
                        throw err;
                    }
                }
            }
            if (exists) {
                return this.updateDoc(props);
            }
            else {
                return this.createDoc(props);
            }
        });
    }
    _archiveDoc({ form, id, reorder }) {
        return __awaiter(this, void 0, void 0, function* () {
            this._clearCache();
            const response = yield this._request({
                form,
                promiseFactory: (appId) => {
                    return this._registrar.client.record.archive({
                        appId,
                        componentName: this.get('storeName'),
                        id,
                    });
                },
            });
            if (reorder) {
                // Remove from ordered list and reorder affected docs. TODO: should we refactor archive so
                // that you can pass in values like order? This way, you can archive and reorder in a single
                // call.
                //
                // TODO: we don't await the _updateDoc() here as it can take a while to reorder all the docs
                // and we don't want it to take this long to confirm the deletion. Is there a better way? We
                // could:
                //   - Pass the where property to the back end so that the back end can perform the
                //     reordering. Would this be fast enough? The back end would still have to iterate through
                //     all the items.
                //   - Support batch updates. I'm not clear how much faster this would be though as we are
                //     already using web sockets and so there is less overhead per update.
                //   - If we stick with this detached construct (no await) then we should probably report any
                //     errors via an 'updateErr' event on the store
                this._updateDoc({
                    form,
                    id,
                    order: reorder_1.Reorder.DEFAULT_ORDER,
                    reorder: true,
                    fieldValues: {},
                });
            }
            return response.data.archiveRecord;
        });
    }
    _restoreDoc({ form, id, order, reorder }) {
        return __awaiter(this, void 0, void 0, function* () {
            this._clearCache();
            const response = yield this._request({
                form,
                promiseFactory: (appId) => {
                    return this._registrar.client.record.restore({
                        appId,
                        componentName: this.get('storeName'),
                        id,
                    });
                },
            });
            if (this._shouldSetToLastOrder(order, reorder)) {
                // Reorder and get the lastOrder so that we can move the restored doc to the end of the list
                const result = yield this._reorder({ form, id });
                yield this._updateDoc({
                    form,
                    id,
                    fieldValues: {},
                    order: result.lastOrder,
                    reorder: false,
                });
            }
            return response.data.restoreRecord;
        });
    }
}
exports.default = RecordStore;
// Used to paginate the data when reordering so that we break up our batches
RecordStore.ITEMS_PER_PAGE_DEFAULT = 50;
