"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.shouldMove = exports.shouldGetAll = exports.updateDoc = exports.createDoc = exports.shouldCRUD = exports.createForm = void 0;
const form_1 = __importDefault(require("../form"));
const fields_1 = require("../fields");
const test_utils_1 = __importDefault(require("../utils/test-utils"));
const reorder_1 = require("./reorder");
const createForm = (props) => {
    return new form_1.default(Object.assign({ fields: [
            new fields_1.TextField({ name: 'firstName' }),
            new fields_1.TextField({ name: 'lastName' }),
        ] }, props));
};
exports.createForm = createForm;
const shouldCRUD = (Store, props) => __awaiter(void 0, void 0, void 0, function* () {
    let fieldValues = {
        firstName: 'Ella',
        lastName: 'Fitzgerald',
    };
    const form = (0, exports.createForm)({
        value: fieldValues,
    });
    const store = new Store(props);
    // Create
    const created = yield store.createDoc({ form });
    expect(created.id).not.toBeFalsy();
    expect(created.archivedAt).toBeNull();
    expect(created.createdAt).not.toBeFalsy();
    expect(created.updatedAt).not.toBeFalsy();
    expect(created.fieldValues).toEqual(fieldValues);
    expect(created.order).toEqual(reorder_1.Reorder.DEFAULT_ORDER);
    // Get
    expect(yield store.getDoc({ id: created.id })).toEqual(created);
    // Get when empty
    expect(yield store.getDoc({ where: { id: 'missingId' } })).toBeNull();
    form.setValues({
        id: created.id,
        firstName: 'F. Scott',
    });
    // Make sure timestamps aren't the same
    yield test_utils_1.default.sleepToEnsureDifferentTimestamps();
    // Update
    const updated = yield store.updateDoc({ id: created.id, form });
    expect(updated).toEqual({
        id: created.id,
        archivedAt: null,
        createdAt: created.createdAt,
        updatedAt: updated.updatedAt,
        userId: null,
        fieldValues: {
            firstName: 'F. Scott',
            lastName: 'Fitzgerald',
        },
        order: reorder_1.Reorder.DEFAULT_ORDER,
    });
    expect(updated.updatedAt).not.toEqual(created.updatedAt);
    expect(yield store.getDoc({ id: created.id })).toEqual(updated);
    // Make sure timestamps aren't the same
    yield test_utils_1.default.sleepToEnsureDifferentTimestamps();
    // Archive
    const archived = yield store.archiveDoc({ id: created.id });
    expect(archived).toEqual(Object.assign({}, updated, {
        archivedAt: archived.archivedAt,
        updatedAt: archived.updatedAt,
        order: reorder_1.Reorder.DEFAULT_ORDER,
    }));
    expect(archived.archivedAt).not.toBeFalsy();
    expect(archived.updatedAt).not.toEqual(updated.updatedAt);
    // Make sure timestamps aren't the same
    yield test_utils_1.default.sleepToEnsureDifferentTimestamps();
    // Restore
    const restored = yield store.restoreDoc({ id: created.id });
    expect(restored).toEqual(Object.assign({}, updated, {
        updatedAt: restored.updatedAt,
        order: reorder_1.Reorder.DEFAULT_ORDER,
    }));
    expect(restored.archivedAt).toBeNull();
    expect(restored.updatedAt).not.toEqual(archived.updatedAt);
});
exports.shouldCRUD = shouldCRUD;
const createDoc = (store, fieldValues) => __awaiter(void 0, void 0, void 0, function* () {
    const form = (0, exports.createForm)({ value: fieldValues });
    return store.createDoc({ form, reorder: true });
});
exports.createDoc = createDoc;
const createDocs = (store) => __awaiter(void 0, void 0, void 0, function* () {
    const harryValues = {
        firstName: 'Harry',
        lastName: 'Potter',
        order: 0,
    };
    const harry = yield (0, exports.createDoc)(store, harryValues);
    const hermioneValues = {
        firstName: 'Hermione',
        lastName: 'Granger',
        order: 1,
    };
    let hermione = yield (0, exports.createDoc)(store, hermioneValues);
    hermione = yield store.archiveDoc({ id: hermione.id });
    const ronValues = {
        firstName: 'Ron',
        lastName: 'Weasley',
    };
    const ron = yield (0, exports.createDoc)(store, ronValues);
    return { harry, hermione, ron };
});
const updateDoc = (store, fieldValues) => __awaiter(void 0, void 0, void 0, function* () {
    const form = (0, exports.createForm)({ value: fieldValues });
    return store.updateDoc({ form, reorder: true });
});
exports.updateDoc = updateDoc;
const searchDefaults = {
    after: null,
    before: null,
    first: null,
    order: null,
    showArchived: null,
};
const shouldGetAll = (Store, props) => __awaiter(void 0, void 0, void 0, function* () {
    const store = new Store(props);
    const { harry, hermione, ron } = yield createDocs(store);
    const all = {
        pageInfo: {
            hasNextPage: false,
        },
        edges: [
            {
                node: harry,
            },
            {
                node: ron,
            },
            {
                node: hermione,
            },
        ],
    };
    // Default props
    expect(yield store.getAllDocs(Object.assign(Object.assign({}, searchDefaults), { showArchived: null }))).toEqual(all);
    // Search
    expect(yield store.getAllDocs(Object.assign(Object.assign({}, searchDefaults), { where: {
            $and: [
                {
                    $or: [
                        { 'fieldValues.firstName': { $iLike: 'h%' } },
                        { 'fieldValues.lastName': { $iLike: 'h%' } },
                    ],
                },
            ],
        } }))).toEqual(Object.assign({}, all, { edges: [{ node: harry }, { node: hermione }] }));
    // Archived status
    expect(yield store.getAllDocs(Object.assign(Object.assign({}, searchDefaults), { showArchived: true }))).toEqual(Object.assign({}, all, { edges: [{ node: hermione }] }));
    expect(yield store.getAllDocs(Object.assign(Object.assign({}, searchDefaults), { showArchived: false }))).toEqual(Object.assign({}, all, { edges: [{ node: harry }, { node: ron }] }));
    // Order
    expect(yield store.getAllDocs(Object.assign(Object.assign({}, searchDefaults), { order: [['fieldValues.firstName', 'DESC']] }))).toEqual(Object.assign({}, all, {
        edges: [{ node: ron }, { node: hermione }, { node: harry }],
    }));
});
exports.shouldGetAll = shouldGetAll;
const expectDocsToEqual = (docs, items) => {
    const edges = [];
    items.forEach((item) => {
        edges.push({
            node: {
                fieldValues: {
                    firstName: item.firstName,
                },
                order: item.order,
            },
        });
    });
    expect(docs).toMatchObject({ edges });
};
const shouldMove = (Store, props) => __awaiter(void 0, void 0, void 0, function* () {
    const store = new Store(props);
    let { harry, hermione, ron } = yield createDocs(store);
    const all = {
        pageInfo: {
            hasNextPage: false,
        },
        edges: [
            {
                node: harry,
            },
            {
                node: ron,
            },
            {
                node: hermione,
            },
        ],
    };
    // Initial ordering
    expect(yield store.getAllDocs(searchDefaults)).toEqual(all);
    // Create
    const ginnyValues = {
        firstName: 'Ginny',
        lastName: 'Weasley',
        order: 1,
    };
    const ginny = yield (0, exports.createDoc)(store, ginnyValues);
    expectDocsToEqual(yield store.getAllDocs(searchDefaults), [
        {
            firstName: 'Harry',
            order: 0,
        },
        {
            firstName: 'Ginny',
            order: 1,
        },
        {
            firstName: 'Ron',
            order: 2,
        },
        {
            firstName: 'Hermione',
            order: reorder_1.Reorder.DEFAULT_ORDER,
        },
    ]);
    // Move up
    ron = yield (0, exports.updateDoc)(store, Object.assign({}, ron.fieldValues, { id: ron.id, order: 0 }));
    expectDocsToEqual(yield store.getAllDocs(searchDefaults), [
        {
            firstName: 'Ron',
            order: 0,
        },
        {
            firstName: 'Harry',
            order: 1,
        },
        {
            firstName: 'Ginny',
            order: 2,
        },
        {
            firstName: 'Hermione',
            order: reorder_1.Reorder.DEFAULT_ORDER,
        },
    ]);
    // Move to end of ordered list
    harry = yield (0, exports.updateDoc)(store, Object.assign({}, harry.fieldValues, { id: harry.id, order: 2 }));
    expectDocsToEqual(yield store.getAllDocs(searchDefaults), [
        {
            firstName: 'Ron',
            order: 0,
        },
        {
            firstName: 'Ginny',
            order: 1,
        },
        {
            firstName: 'Harry',
            order: 2,
        },
        {
            firstName: 'Hermione',
            order: reorder_1.Reorder.DEFAULT_ORDER,
        },
    ]);
    // Archive
    yield store.archiveDoc({ id: ginny.id, reorder: true });
    expectDocsToEqual(yield store.getAllDocs(searchDefaults), [
        {
            firstName: 'Ron',
            order: 0,
        },
        {
            firstName: 'Harry',
            order: 1,
        },
        {
            firstName: 'Hermione',
            order: reorder_1.Reorder.DEFAULT_ORDER,
        },
        {
            firstName: 'Ginny',
            order: reorder_1.Reorder.DEFAULT_ORDER,
        },
    ]);
    // Restore
    yield store.restoreDoc({ id: ginny.id, reorder: true });
    expectDocsToEqual(yield store.getAllDocs(searchDefaults), [
        {
            firstName: 'Ron',
            order: 0,
        },
        {
            firstName: 'Harry',
            order: 1,
        },
        {
            firstName: 'Ginny',
            order: 2,
        },
        {
            firstName: 'Hermione',
            order: reorder_1.Reorder.DEFAULT_ORDER,
        },
    ]);
});
exports.shouldMove = shouldMove;
