"use strict";
// Highlights:
// - deepCloneWith() is more than 2 times faster than Lodash's cloneDeepWith()! See
//   deep-clone.test.js
// - deepClone() allows undefined values to be cloned, e.g.
//     deepClone({ foo: undefined }) = { foo: undefined }
//   but:
//     cloneDeep({ foo: undefined }) = {}
Object.defineProperty(exports, "__esModule", { value: true });
exports.cloneDeep = exports.cloneDeepWith = exports.deepClone = exports.deepCloneWith = void 0;
const isClass = (object) => {
    return object.constructor && object.constructor.name !== 'Object';
};
const deepCloneWithInner = (object, onNode, key, stack) => {
    if (stack.indexOf(object) !== -1) {
        // Bail if we find a circular reference
        return object;
    }
    let doClone = true;
    let clonedVal = object;
    if (onNode) {
        const { performDefaultClone, clonedObject } = onNode(object, key, object, stack);
        doClone = performDefaultClone;
        if (!doClone) {
            clonedVal = clonedObject;
        }
    }
    if (doClone) {
        if (object === null) {
            return null;
        }
        else if (Array.isArray(object)) {
            stack.push(object);
            clonedVal = object.map((value, key) => deepCloneWithInner(value, onNode, key, stack));
            stack.pop();
        }
        else if (typeof object === 'object') {
            stack.push(object);
            if (isClass(object)) {
                // Note: this works for both classes and other objects, but it is much slower than `clonedVal = {}`
                clonedVal = Object.assign(Object.create(Object.getPrototypeOf(object)), object);
            }
            else {
                clonedVal = {};
            }
            for (let key in object) {
                clonedVal[key] = deepCloneWithInner(object[key], onNode, key, stack);
            }
            stack.pop();
        }
    }
    return clonedVal;
};
const deepCloneWith = (object, onNode) => deepCloneWithInner(object, onNode, undefined, []);
exports.deepCloneWith = deepCloneWith;
const deepClone = (object) => deepCloneWithInner(object, undefined, undefined, []);
exports.deepClone = deepClone;
// Drop-in replacement for Lodash's cloneDeepWith()
const cloneDeepWith = (object, customizer) => (0, exports.deepCloneWith)(object, (value, key, object, stack) => {
    const clonedObject = customizer(value, key, object, stack);
    if (clonedObject === undefined) {
        return {
            performDefaultClone: true,
        };
    }
    else {
        return {
            performDefaultClone: false,
            clonedObject,
        };
    }
});
exports.cloneDeepWith = cloneDeepWith;
// Drop-in replacement for Lodash's cloneDeep()
const cloneDeep = (object) => (0, exports.deepClone)(object);
exports.cloneDeep = cloneDeep;
