"use strict";
// TODO: move to ../test and rename to just utils.js
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const compiler_1 = __importDefault(require("../compiler"));
const form_1 = __importDefault(require("../form"));
const utils_1 = __importDefault(require("./utils"));
const component_1 = __importDefault(require("../component/component"));
// Throw action errors as we should not receive them in our test environment. Note: we cannot toggle
// this option in a globalSetup as the scope in a globalSetup is not shared with the tests.
component_1.default.setThrowActionErrors(true);
class TestUtils {
    constructor() {
        this.defaultFields = [
            'id',
            'userId',
            'createdAt',
            'updatedAt',
            'archivedAt',
            'order',
        ];
    }
    toDefaultFieldsObject(value) {
        const obj = {};
        this.defaultFields.forEach((name) => (obj[name] = value));
        return obj;
    }
    once(emitter, evnt) {
        return utils_1.default.once(emitter, evnt);
    }
    timeout(ms) {
        return new Promise((resolve) => {
            setTimeout(resolve, ms);
        });
    }
    // TODO: refactor to use await/async
    waitFor(poll, maxSleep, sleepMs) {
        var totalSleep = 0;
        maxSleep = maxSleep ? maxSleep : 5000;
        sleepMs = sleepMs ? sleepMs : 100;
        return new Promise(function (resolve, reject) {
            var waitFor = function () {
                // Wrap in promise so that waitMore doesn't have to be a promise
                return Promise.resolve()
                    .then(function () {
                    return poll();
                })
                    .then(function (obj) {
                    if (typeof obj === 'undefined') {
                        if (totalSleep >= maxSleep) {
                            reject(new Error('waited for ' + totalSleep + ' ms'));
                        }
                        else {
                            totalSleep += sleepMs;
                            setTimeout(waitFor, sleepMs);
                        }
                    }
                    else {
                        resolve(obj);
                    }
                });
            };
            waitFor();
        });
    }
    _expectValuesToHaveErr(field, values, haveErr, err) {
        values.forEach((value) => {
            field.clearErr();
            field.setValue(value);
            field.validate();
            // Use object in expect so that errors are easy to immediately see
            expect({
                value,
                hasErr: field.hasErr(),
            }).toEqual({
                value,
                hasErr: haveErr,
            });
            if (err) {
                expect(field.getErr()).toEqual(err);
            }
        });
    }
    expectValuesToBeValid(field, validValues) {
        this._expectValuesToHaveErr(field, validValues, false);
    }
    expectValuesToBeInvalid(field, invalidValues, err) {
        this._expectValuesToHaveErr(field, invalidValues, true, err);
    }
    expectSchemaToBeValid(component, values) {
        const schemaForm = new form_1.default();
        component.buildSchemaForm(schemaForm, compiler_1.default);
        schemaForm.setValues(values);
        schemaForm.validate();
        expect(schemaForm.hasErr()).toEqual(false);
    }
    // Note: this doesn't appear to work so we use the implementation below
    // async expectToFinishBefore(promiseFactory, milliseconds) {
    //   const timer = async () => {
    //     await testUtils.timeout(milliseconds);
    //     throw new Error('took more than ' + milliseconds + ' milliseconds');
    //   }
    //   return Promise.race([timer(), promiseFactory()])
    // }
    // Note: the built in timeout per test in jest doesn't appear to work so here is a custom routine.
    expectToFinishBefore(promiseFactory, milliseconds) {
        return __awaiter(this, void 0, void 0, function* () {
            const begin = new Date();
            yield promiseFactory();
            if (new Date().getTime() - begin.getTime() > milliseconds) {
                throw new Error('took more than ' + milliseconds + ' milliseconds');
            }
        });
    }
    sleepToEnsureDifferentTimestamps() {
        return __awaiter(this, void 0, void 0, function* () {
            // Sleep for 2 milliseconds as timestamps can be the same with 1 millisecond
            return this.timeout(2);
        });
    }
    mockActionGlobals(action, mockedGlobals) {
        action._componentFillerProps._getGlobals = () => mockedGlobals;
        action._globals = mockedGlobals;
    }
    mockActionRegistrar(action, mockedRegistrar) {
        // TODO: uncomment when used. Commented out for test coverage reasons
        // action._componentFillerProps._registrar = () => mockedRegistrar;
        action._registrar = mockedRegistrar;
    }
    mockActions(actions, acts, stub, mockedGlobals, mockedRegistrar) {
        actions.forEach((action) => {
            const actions = action._getProperty('actions');
            const elseActions = action._getProperty('else');
            if (actions || elseActions) {
                // if (actions) { // TODO: uncomment if needed
                this.mockActions(actions, acts, stub, mockedGlobals, mockedRegistrar);
                // }
                if (elseActions) {
                    this.mockActions(elseActions, acts, stub, mockedGlobals, mockedRegistrar);
                }
            }
            else {
                const origAct = action.act;
                const spy = jest.fn();
                action.act = function () {
                    acts.push({
                        name: action.getClassName(),
                        props: action.get(),
                        spy,
                    });
                    spy.apply(spy, arguments);
                    if (!stub) {
                        return origAct.apply(this, arguments);
                    }
                };
            }
            if (mockedGlobals) {
                this.mockActionGlobals(action, mockedGlobals);
            }
            if (mockedRegistrar) {
                this.mockActionRegistrar(action, mockedRegistrar);
            }
        });
    }
    mockAction(action, acts, stub, mockedGlobals, mockedRegistrar) {
        this.mockActions([action], acts, stub, mockedGlobals, mockedRegistrar);
    }
    expectActsToContain(acts, expActs) {
        expect(acts).toHaveLength(expActs.length);
        expActs.forEach((expAct, i) => {
            const act = acts[i];
            const actualProps = {};
            if (expAct.props !== undefined) {
                Object.keys(expAct.props).forEach((name) => {
                    actualProps[name] = act.props[name];
                });
            }
            const actualAct = { name: act.name };
            if (expAct.props !== undefined) {
                actualAct.props = actualProps;
            }
            expect(actualAct).toEqual(expAct);
        });
    }
    mockComponentListeners(component, acts, event, stub, mockedGlobals, mockedRegistrar) {
        const listeners = component.get('listeners');
        listeners.forEach((listener) => {
            if (!event || listener.event === event) {
                this.mockActions(listener.actions, acts, stub, mockedGlobals, mockedRegistrar);
            }
        });
    }
}
exports.default = new TestUtils();
