"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UberUtils = void 0;
const globals_1 = __importDefault(require("../globals"));
// Utils that are above the component layer
class UberUtils {
    constructor() {
        // For mocking
        this._globals = globals_1.default;
    }
    displayError(text) {
        this._globals.displayAlert({ title: 'Unexpected Error', text });
    }
    setFormErrorsFromAPIError(err, form) {
        try {
            const message = JSON.parse(err.graphQLErrors[0].message);
            message.error.forEach((err) => {
                form.getField(err.field).setErr(err.error);
            });
        }
        catch (_err) {
            // An error can occur if the message is not a JSON object, e.g. if we don't have access to
            // archive, etc... The caller will still throw the main error. TODO: Is there a better way to
            // handle this? Should all messages be JSON objects? That would probably be too limiting,
            // right?
            this.displayError(err.toString());
        }
    }
    tryAndSetFormErrorsIfAPIError(promiseFactory, form) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield promiseFactory();
            }
            catch (err) {
                this.setFormErrorsFromAPIError(err, form);
                // We throw the error so that the entire listener chain is aborted
                throw err;
            }
        });
    }
    tryAndDisplayErrorIfAPIError(promiseFactory) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield promiseFactory();
            }
            catch (err) {
                this.displayError(err.toString());
                // We throw the error so that the entire listener chain is aborted
                throw err;
            }
        });
    }
}
exports.UberUtils = UberUtils;
exports.default = new UberUtils();
