"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Utils = void 0;
const uuid_1 = require("uuid");
class Utils {
    constructor() {
        // For mocking
        this._global = global;
    }
    _reduce(collection, callback, accumulator) {
        if (Array.isArray(collection)) {
            return collection.reduce(callback, accumulator);
        }
        else {
            return Object.entries(collection).reduce((previousValue, [key, item]) => {
                return callback(previousValue, item, key);
            }, accumulator);
        }
    }
    sequential(items, onItem) {
        return __awaiter(this, void 0, void 0, function* () {
            const values = [];
            yield this._reduce(items, (promise, item, key) => __awaiter(this, void 0, void 0, function* () {
                yield promise;
                const value = yield onItem(item, key);
                if (value !== undefined) {
                    values.push(value);
                }
            }), Promise.resolve());
            return values;
        });
    }
    // // A recursive merge that also concats arrays
    // merge(object, sources) {
    //   const mergeCustomizer = (objValue, srcValue) => {
    //     if (_.isArray(objValue)) {
    //       return objValue.concat(srcValue);
    //     }
    //   };
    //   return _.mergeWith(object, sources, mergeCustomizer);
    // }
    inBrowser() {
        return !!this._global.window;
    }
    // Source: https://stackoverflow.com/a/40577337/2831606
    getAllMethodNames(obj) {
        const methods = [];
        // Note: this silly function is needed to prevent eslint from complaining about "Function
        // declared in a loop contains unsafe references to variable(s)"
        const fun = (obj) => {
            return (k) => {
                // Prevent considering things like symbols
                if (typeof obj[k] === 'function') {
                    methods.push(k);
                }
            };
        };
        while ((obj = Reflect.getPrototypeOf(obj))) {
            const keys = Reflect.ownKeys(obj);
            keys.forEach(fun(obj));
        }
        return methods;
    }
    getAllFunctionNames(obj) {
        const methods = [];
        Object.entries(obj).forEach(([name, property]) => {
            if (typeof property === 'function') {
                methods.push(name);
            }
        });
        return methods;
    }
    combineWheres(...wheres) {
        const and = [];
        wheres.forEach((where) => {
            if (where) {
                and.push(where);
            }
        });
        if (and.length === 1) {
            return and[0];
        }
        else {
            return {
                $and: and,
            };
        }
    }
    toWhereFromSearchString(attributes, string) {
        const trimmed = string.trim();
        // Empty?
        if (trimmed === '') {
            return null;
        }
        const words = trimmed.split(/ +/);
        const ands = [];
        words.forEach((word) => {
            const ors = [];
            attributes.forEach((attr) => {
                ors.push({
                    [attr]: {
                        // We need to use iLike as like is not case sensitive with binary (like JSON) data
                        $iLike: word + '%',
                    },
                });
            });
            ands.push({
                $or: ors,
            });
        });
        return {
            $and: ands,
        };
    }
    uuid() {
        return (0, uuid_1.v4)();
    }
    once(emitter, evnt) {
        return new Promise(function (resolve) {
            emitter.once(evnt, function () {
                resolve(arguments);
            });
        });
    }
    isRegExpString(string) {
        return new RegExp('^/(.*)/(.*)$').test(string);
    }
    stringToRegExp(string) {
        // JSON doesn't support RegExp types so convert string representations to RegExp, including
        // flags
        const match = string.match(new RegExp('^/(.*)/(.*)$'));
        return new RegExp(match[1], match[2]);
    }
    toRegExp(item) {
        return item instanceof RegExp ? item : this.stringToRegExp(item);
    }
    toSingular(plural) {
        // Automatically calculate singular label by removing last 's'
        return plural.substr(0, plural.length - 1);
    }
    // synchronize(synchronizer, promiseFactory) {
    //   const promise = synchronizer.then(() => {
    //     return promiseFactory();
    //   });
    //   synchronizer = synchronizer.then(() => promise);
    //   return promise;
    // }
    // Credit: https://github.com/you-dont-need/You-Dont-Need-Lodash-Underscore#_difference
    difference(a, b) {
        if (b === undefined) {
            return a;
        }
        else {
            return [a, b].reduce((a, b) => a.filter((c) => !b.includes(c)));
        }
    }
    // Note: only use this if you need to exit the loop prematurely by having onItem() return false;
    // or if obj may be falsy. Otherwise, just use Object.entries(), Object.keys(), or
    // Object.values().
    each(obj, onItem) {
        if (obj !== undefined && obj !== null) {
            const entries = Object.entries(obj);
            for (let i = 0; i < entries.length; i++) {
                const [key, value] = entries[i];
                const again = onItem(value, key);
                if (again === false) {
                    break;
                }
            }
        }
    }
}
exports.Utils = Utils;
exports.default = new Utils();
